<?php
/*
Plugin Name: RYO Category Visibility
Plugin URI: http://ryowebsite.com/wp-plugins
Description: Alter the visibility settings for categories shown in the front page, sidebar. For WordPress 2.8+.
Author: Rich Hamilton
Version: 3.0.5
Author URI: http://ryowebsite.com
*/
/*

v. 3.0   Faster logic and better compatibility than our earlier Category Visibility.
Based on an earlier concept by Keith McDuffee but with entirely new logic.
The old version maintained a separate data table, changed joins and wheres,
but this one just tells WP what NOT to show.

*/
/*
============================================================================================================
    Copyright (C) 2009  Rich Hamilton (http://ryowebsite.com)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

============================================================================================================ 
    Plugin Update Checker Library 1.0 is independent of
    the main plugin and is licensed under the GNU GPL license.
    full details can be found at http://www.gnu.org/licenses/gpl.html
    The library remains copyright 2011 Janis Elsts
    http://w-shadow.com/
============================================================================================================ 
*/


if (!function_exists('ryo_plugindata')) {
    function ryo_plugindata($plugin_file,$gget='version') {
        // Must be located in the main plugin file or pass plugin file 
        if ( !function_exists('get_plugins') ) { require_once(ABSPATH . 'wp-admin/includes/plugin.php'); }
        $plugin_file=plugin_basename($plugin_file); // sellbetter_toolbox_api/sbt_query.php
        $plugin_folder=get_plugins();
        $aget=array('Version','Name','Title','PluginURI','Description','Author','AuthorName','AuthorURI','TextDomain','DomainPath','Network');
        foreach ($aget as $a) { if (strcasecmp($a,$gget)==0) return $plugin_folder[$plugin_file][$a];  }
        return $plugin_folder[$plugin_file]['Version'];
    }
}

require 'plugin-updates/plugin-update-checker.php';
$ryocatvis_UpdateChecker = PucFactory::buildUpdateChecker(
    'http://razataz.com.S3.amazonaws.com/plugins/ryo_catvis.json',
    __FILE__,
    'ryo_catvis',
    12
);

function ryocatvis_posts($qobj) {
    //if ( 'posts' == get_option('show_on_front') && $qobj->is_home ) $cycle = 'front';
    if (isset($qobj->queried_object_id) && $qobj->queried_object_id == get_option('page_for_posts'))
        $cycle = 'front';                 // Posts page
    elseif ( $qobj->is_home ) $cycle = 'front';  
    elseif ( $qobj->is_tag ) $cycle = 'tags';
    elseif ( $qobj->is_archive && !$qobj->is_category ) $cycle = 'archives';
    elseif ( $qobj->is_search && !$qobj->is_admin ) $cycle = 'search';
    elseif ( $qobj->is_feed ) $cycle = 'feed';
    else $cycle = '';
    if ($cycle) {
        $op = ryocatvis_vals();
        $orig = (isset($qobj->query_vars['cat'])) ? (array) explode(',',$qobj->query_vars['cat']) : array();
        $cats = array_filter(ryocatvis_excludelevels(array_merge($orig,$op[$cycle]),$op['level']));
        $cats = (count($cats)>0) ? '-'.implode(',-',$cats) : '';
        if (strlen($cats)>1) {
            $qobj->parsed_tax_query=false;
            $qobj->query_vars=array_merge( (array)$qobj->query_vars, array('cat' => urlencode($cats)) );
        }
        if ($cycle!='search' && $op['postmash'] && function_exists('postMash_main')) {
            $qobj->parsed_tax_query=false;
            $qobj->query_vars=array_merge( (array)$qobj->query_vars, array('orderby' => 'menu_order', 'order' => 'ASC') );
        }
    }
}
add_action('pre_get_posts','ryocatvis_posts',10,1);

function ryocatvis_lists($terms, $taxonomies, $args){
    if (isset($args['get']) && $args['get']=='all') return $terms;
    if (in_array('category',$taxonomies) && !is_admin()) {
        $ret = array();
        $op = ryocatvis_vals();
        $skipcats = ryocatvis_excludelevels($op['list'],$op['level']);
        if($skipcats){
            foreach ($terms as $key => $term) {
                if (!in_array($term->term_id,$skipcats)) $ret[$key]=$term;
            }
            return $ret;
        }
    }
    return $terms;
}
add_filter('get_terms', 'ryocatvis_lists',10,3);

function ryocatvis_archivelist($where,$r) {
    global $wpdb;
    // Get array of exclude cats
    $op = ryocatvis_vals();
    $skipcats = ryocatvis_excludelevels($op['archives'],$op['level']);
    if ($skipcats) {
        $cat_string = "'" . implode("', '", $skipcats) . "'";
        $where .= " AND $wpdb->posts.ID NOT IN ( SELECT tr.object_id FROM $wpdb->term_relationships AS tr INNER JOIN $wpdb->term_taxonomy AS tt ON tr.term_taxonomy_id = tt.term_taxonomy_id WHERE tt.taxonomy = 'category' AND tt.term_id IN ($cat_string) )";
    }
    return $where;
}
add_filter('getarchives_where','ryocatvis_archivelist',10,2);

function ryocatvis_excludelevels($exclude,$oplevels) {
    global $user_level;
    require_once(ABSPATH . WPINC . '/pluggable.php');
    get_currentuserinfo();
    if (!is_numeric($user_level)) $user_level = 0;
    if (!is_array($exclude)) $exclude=array();
    foreach ((array)$oplevels as $cat => $level) {
        if ($level > $user_level && !in_array($cat,$exclude)) $exclude[] = $cat;
    }
    return $exclude;
}

function ryocatvis_vals() {
    $op = get_settings('ryocatvis');
    if (!is_array($op)) $op=array('exclude'=>0,'postmash'=>0,'level'=>array());
    $op['cycles']=array('front','list','search','feed','archives','tags');
    foreach ($op['cycles'] as $cycle) { if (!isset($op[$cycle]) || !is_array($op[$cycle])) $op[$cycle]=array();  }
    return $op;
}

function ryocatvis_options_page() {
    global $wp_roles;
    $op = ryocatvis_vals();
    $cycles=$op['cycles'];
    $categories = get_categories("child_of=0&hide_empty=0&hierarchical=1");
    $categories = ryocatvis_cat_tree($categories);
    if ( isset($_POST['submitted']) ) {
        check_admin_referer('ryocatvis_options_all');
        $op = array();
        $op['exclude'] = (isset($_POST['exclude']) && $_POST['exclude']) ? 1 : 0;
        $op['postmash'] = (isset($_POST['postmash']) && $_POST['postmash']) ? 1 : 0;
        $op['level'] = (isset($_POST['level']) && $_POST['level']) ? $_POST['level'] : array();
        foreach ($cycles as $cycle) {
            $afront = (array) $_POST[$cycle];
            $op[$cycle]=array();
            foreach ($categories as $category) {
                if ( ($op['exclude'] && isset($afront[$category->cat_ID])) 
                || (!$op['exclude'] && !isset($afront[$category->cat_ID])) ) $op[$cycle][]=$category->cat_ID;
            }
        }
        update_option('ryocatvis', $op);
        echo '<div id="message" class="updated fade"><p><strong>Plugin settings saved.</strong></p></div>';
    }
    $style = " scope='col' style='padding:2px 10px;background-color:#ececec;'";
?>
<script type="text/javascript">
    <!--
    function ryocatvisConvertHead(checked) {
        if (checked) {document.getElementById('ExclHead').innerHTML='<?php _e('Exclude'); ?>';};
        if (!checked) {document.getElementById('ExclHead').innerHTML='<?php _e('Visibility'); ?>';};
    }
    -->
</script>        
<style>
    cite {margin:5px 0 0 0;font-size:100%;font-weight:normal;}
    .rsswidget, .rsswidget a {font-size:100%;font-weight:bold;}
    .rss-date {font-size:85%;color:#999999;font-weight:normal;font-style:italic;}
    .rssSummary {font-size:100%;font-weight:normal;}
</style>
<div class='wrap'>
    <h2><?php echo ryo_plugindata(__FILE__,'name'); ?></h2>
    <p>Select which categories you want to include (or exclude) for various parts of your website.
    </p>
    <?php if (function_exists('postMash_main')) { ?>
        <p>Since you're using postMash, you may opt to use our function to have pages sort posts<br>automatically according to the postMash order (without template modifications).</p>
    <?php } ?>
    <form name="catpostshomepage" action="" method="post">
        <?php if (function_exists('wp_nonce_field')) { wp_nonce_field('ryocatvis_options_all'); } ?>
        <input type="hidden" name="submitted" value="1" />

        <table class="widefat" style="width:800px;">
            <thead>
                <tr><th class="handlediv" style="width:100%;border-right:1px #cccccc solid" colspan="3">
                        <b><?php echo ryo_plugindata(__FILE__,'name') . ' v. '.ryo_plugindata(__FILE__); ?></b>
                    </th></tr></thead>
            <tbody>
            <tr><td style="background-color:#ffffff;padding:10px;border-right:1px #cccccc solid;vertical-align:top;" colspan="3">
                    <ul>
                        <?php $thisval= ($op['exclude']) ? "checked='checked'" : ''; 
                        if (!$op['exclude']) {
                            echo '<input type="hidden" id="exclude" name="exclude" value="0">';
                        } else {
                        ?>
                        <li class='popular-category'><input type='checkbox' id='exclude' name='exclude' onclick='ryocatvisConvertHead(this.checked)' <?php echo $thisval; ?> />
                            &nbsp;<label for='exclude' class='selectit'>
                                Check to exclude selected categories. (Leave unchecked to include selected categories.)</label>
                        </li>
                        <?php 
                        }
                            if (!function_exists('postMash_main')) {
                                echo "\n<input type='hidden' name='postmash' value='{$op['postmash']}' />";
                            } else {
                                $thisval= ($op['postmash']) ? "checked='checked'" : ''; ?>
                            <li class='popular-category'><input type='checkbox' id='postmash' name='postmash' <?php echo $thisval; ?> />
                                &nbsp;<label for='postmash' class='selectit'>
                                    Check to sort posts in postMash order rather than date order.)</label>
                            </li>
                            <?php                    
                            }
                        ?>
                    </ul>
                    <b>Select Categories, then click 'Save Changes':</b>
                </td></tr>
            <tr>
                <th<?php echo $style; ?>><?php _e('ID'); ?></th>
                <th<?php echo $style; ?>><?php _e('Name'); ?></th>
                <th<?php echo $style; ?> id='ExclHead'><?php echo ($op['exclude']) ? __('Exclude') : __('Visibility'); ?></th>
            </tr>
            <?php
                $alternate = false;
                foreach ($categories as $category) {
                    $edit = '';
                    foreach ($cycles as $cycle) {
                        $labelit = ucfirst($cycle); if ($cycle=='front') $labelit="Home";
                        $ischecked = (($op['exclude'] && in_array($category->cat_ID,$op[$cycle]))
                        || (!$op['exclude'] && !in_array($category->cat_ID,$op[$cycle])) ) ? "checked='checked'" : '';
                        $edit  .= "\n  <label for='{$cycle}[{$category->cat_ID}]'>{$labelit}:</label> <input name='{$cycle}[{$category->cat_ID}]' id='{$cycle}[{$category->cat_ID}]' class='edit' type='checkbox' $ischecked/>&nbsp;&nbsp;";
                    }
                    $levelvalue = (isset($op['level'][$category->cat_ID]) && $op['level'][$category->cat_ID]>0 ) ? $op['level'][$category->cat_ID] : 0;
                    $edit .= "\n<label for='level[{$category->cat_ID}]'>User Level:</label> <input name='level[{$category->cat_ID}]' id='level[{$category->cat_ID}]' class='edit' type='text' size='3' value='$levelvalue' />";
                    $alternate = !$alternate;
                    $style = "style='padding:7px 10px;vertical-align:middle;". (($alternate) ? 'background-color:#ffffff;\'' : 'background-color:#ececec;\'');
                    echo "\n<tr><th scope='row' $style>$category->cat_ID</th><td $style><b>$category->ryocatvis_dash".apply_filters('list_cats', $category->cat_name, $category)."</td>
                    <td  $style>$edit</td>
                    </tr>";
                }
            ?>
            <tr><td colspan="3" class="submit" align="center">
                    <input name="Submit" class="button-primary" value="<?php _e('Save Changes') ?>" type="submit">
                </td></tr>
        </table>
        <br>
        <table class="widefat" style="width:800px;">
            <thead>
                <tr><th class="handlediv" style="width:50%;border-right:1px #cccccc solid">
                        <b>Definitions</b>
                    </th>
                    <th style="width:50%;">
                        <b>AutomationBasics.com News</b>
                    </th></tr></thead>
            <tbody>
            <tr><td style="background-color:#ffffff;padding:10px;border-right:1px #cccccc solid;vertical-align:top;">

                    <strong>Home:</strong> Visibility on the Home Page (or an alternate Page for Posts when using a Static Front Page).<br />
                    <strong>List:</strong> Visibility on the list of categories on the sidebar.<br />
                    <strong>Search:</strong> Visibility in search results.<br />
                    <strong>Feed:</strong> Visibility in RSS/RSS2/Atom feeds.<br />
                    <strong>Archive:</strong> Visibility in archive pages (i.e., calendar links).<br />
                    <strong>Tags:</strong> Visibility in tag archive pages (does not keep tags from widget display).<br />
                    <strong>User Level:</strong> Visibility on user level basis.<br />&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;All users this level and higher can see categories as checked.<br />&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Does not affect feed visibility.<br />
                    <strong>Numeric User Levels:</strong><?php
                        // Get Role List
                        foreach($wp_roles->role_objects as $key => $role) {
                            foreach($role->capabilities as $cap => $grant) {
                                $role_user_level = array_reduce(array_keys($role->capabilities), array('WP_User', 'level_reduction'), 0);
                            }
                            echo '<br />&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;'.ucfirst($role->name).': '.$role_user_level;
                        }
                    ?>

                </td><td style="background-color:#ffffff;padding:10px;vertical-align:top;">    
                    <?php
                        wp_widget_rss_output(array(
                        'link' => 'http://automationbasics.com/',
                        'url' => 'http://automationbasics.com/feed/',
                        'title' => 'AutomationBasics.com News',
                        'items' => 3,
                        'show_summary' => 1,
                        'show_author' => 0,
                        'show_date' => 1));
                    ?>
                </td></tr>
        </table>    


    </form>
</div>
<?php
}
function ryocatvis_cat_tree($categories,$childof=0,$dash='') {
    $ret = array();
    $found = 0;
    foreach ($categories as $cat) {
        if ($cat->parent == $childof) {
            $cat->ryocatvis_dash=$dash;
            $ret[] = $cat;
            $more = ryocatvis_cat_tree($categories,$cat->cat_ID,"&#8212;$dash");
            foreach ($more as $each) {
                $ret[] = $each;
            }
        }
    }
    return $ret;
}

function ryocatvis_adminpage() {
    if (current_user_can('manage_options')) {
        $first = 0;
        if(!defined('RYOTOPMENU')){ 
            define('RYOTOPMENU','ryocatvis_options_page'); 
            add_menu_page('RYO Plugins', 'RYO Plugins', 0, RYOTOPMENU, 'ryocatvis_options_page', str_replace(WP_CONTENT_DIR,WP_CONTENT_URL,dirname(__FILE__)).'/ryowebsite.ico'); 
            $first = 1;
        } 
        // For first submenu 5th param below must be sams as RYOTOPMENU ... addl submenus match 6th param function name
        $param5 = ($first) ? RYOTOPMENU : 'ryocatvis_options_page';
        add_submenu_page(RYOTOPMENU, 'RYO Category Visibility', 'Category Visibility', 'manage_options', $param5, 'ryocatvis_options_page', ''); 
    }
}
if ( is_admin() ) add_action('admin_menu', 'ryocatvis_adminpage');

// Add a Settings link to the Plugins page
function ryocatvis_plugin_action_links( $links, $file ) {
    if ( $file == plugin_basename(__FILE__) )
        array_unshift($links,'<a href="admin.php?page=ryocatvis_options_page">' . __('Settings') . '</a>');
    return $links;
}
add_filter( 'plugin_action_links', 'ryocatvis_plugin_action_links', 10, 2 );

?>
